Check environment variables:

  $ echo "$LANG"
  C
  $ echo "$LC_ALL"
  C
  $ echo "$LANGUAGE"
  C
  $ echo "$TZ"
  GMT
  $ echo "$CDPATH"
  
  $ echo "$COLUMNS"
  80
  $ echo "$GREP_OPTIONS"
  
  $ echo "$CRAMTMP"
  .+ (re)
  $ echo "$TESTDIR"
  */examples (glob)
  $ ls "$TESTDIR"
  bare.t
  empty.t
  env.t
  fail.t
  missingeol.t
  skip.t
  test.t
  $ echo "$TESTFILE"
  env.t
  $ pwd
  */cramtests*/env.t (glob)
